import logging

from cachetools import TTLCache, cachedmethod
from operator import attrgetter
from requests import Session
from typing import MutableMapping

from reader_rbv.exception import BookNotFound
from . import Buku

HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:87.0) Gecko/20100101 Firefox/87.0"
}


class Reader:
    def __init__(
        self,
        username: str,
        password: str,
        base: str = "http://www.pustaka.ut.ac.id/reader/",
        session: Session = Session(),
    ):
        self.username = username
        self.password = password
        self.session = session
        self.base = base
        self.cache: MutableMapping[str, Buku] = TTLCache(10, 600)
        self.logger = logging.getLogger(f"Reader:{username}")
        self.session.headers.update(HEADERS)

    @cachedmethod(attrgetter("cache"))
    def get(self, kode: str) -> Buku:
        params = {"modul": kode}
        self.logger.debug(f"Mencari ketersediaan buku {kode}")
        res = self.session.get(self.base, params=params)
        if not res.ok or not res.text:
            self.logger.warning(f"Buku {kode} tidak ditemukan")
            raise BookNotFound(f"Buku dengan kode {kode} tidak ditemukan")
        else:
            self.logger.debug(f"Buku {kode} ditemukan")
        return Buku(
            kode=kode,
            base=self.base,
            session=self.session,
            username=self.username,
            password=self.password,
        )



import attr
from typing import List


@attr.dataclass(slots=True)
class Text:
    height: int
    hLines: int
    vline: int
    fills: int
    font: int
    text: str

    @classmethod
    def from_data(cls, data: list) -> "Text":
        return cls(
            height=data[0],
            hLines=data[1],
            vline=data[2],
            fills=data[3],
            font=data[4],
            text=data[5],
        )

    @classmethod
    def from_list(cls, data: List[list]) -> List["Text"]:
        results: List["Text"] = list()
        for dat in data:
            results.append(cls.from_data(dat))
        return results

    def __str__(self):
        return self.text



from bs4 import BeautifulSoup, Tag
from requests import Response, Session
from typing import Dict, Optional, Union

from reader_rbv.exception import InvalidCredential, Unreachable


def get_captcha(form: Tag) -> str:
    c: Union[str, int, float] = ""
    ccaptcha: Tag = form.find("input", {"name": "ccaptcha"})
    q: str = ccaptcha.previous.strip().lower().split()
    # 'Berapa hasil dari 3 + 9 ='
    a = q[3]
    n = q[4]
    b = q[5]
    if n == "+":
        c = int(a) + int(b)
    elif n == "-":
        c = int(a) - int(b)
    elif n == "/" or n == ":":
        c = int(a) / int(b)
    elif n == "*" or n == "x":
        c = int(a) * int(b)
    return str(c)


def get_url(
    session: Session,
    url: str,
    username: str,
    password: str,
    params: Optional[Dict[str, str]] = None,
    headers: Optional[Dict[str, str]] = None
) -> Response:
    res = session.get(url, params=params, headers=headers)
    if not res.ok:
        raise Unreachable("RBV tidak dapat dihubungi")
    elif "Login" not in res.text:
        return res
    soup = BeautifulSoup(res.text, "html.parser")
    data = {
        "_submit_check": "1",
        "username": username,
        "password": password,
        "ccaptcha": get_captcha(soup),
        "submit": "Submit",
    }
    headers = {"Referer": res.url}
    res = session.post(url, data=data, params=params, headers=headers)
    if not res.ok:
        raise InvalidCredential("Username / password salah")
    return res



from .font import Font
from .text import Text
from .page import Page
from .modul import Modul
from .buku import Buku

from .reader import Reader

__all__ = [
    "Font",
    "Text",
    "Page",
    "Modul",
    "Buku",
    "Reader",
]



from cachetools import LRUCache, cachedmethod
from operator import attrgetter
from requests import Session
from typing import List, Mapping, MutableMapping, Optional
from urllib.parse import urlencode

from . import Page
from .utils import get_url


def clean_doc(doc: str) -> str:
    # DAFIS.pdf -> DAFIS
    return doc.rstrip(".pdf")


class Modul(Mapping[int, Page]):
    def __init__(
        self,
        nama: str,
        subfolder: str,
        doc: str,
        base: str,
        url: str,
        username: str,
        password: str,
        session: Session,
        max_page: Optional[int] = None,
    ):
        self.nama = nama
        self.subfolder = subfolder
        self.doc = clean_doc(doc)
        self.base = base
        self.url = url
        self.session = session
        self.cache: MutableMapping[int, Page] = LRUCache(100)
        self._max_page = max_page
        self.__username__ = username
        self.__password__ = password
        if not self.max_page:
            self.fetch()

    def __str__(self):
        return self.nama

    def __getitem__(self, key: int) -> Page:
        return self.get_page(key)

    def __iter__(self):
        self.__current__ = 1
        return self

    def __next__(self):
        if self.__current__ <= self.max_page:
            result = self.get_page(self.__current__)
            self.__current__ += 1
            return result
        else:
            raise StopIteration

    def __len__(self):
        return self.max_page

    @property
    def max_page(self) -> Optional[int]:
        return self._max_page

    @max_page.setter
    def max_page(self, value: int):
        if isinstance(value, int):
            self._max_page = value
        raise ValueError("max_page harus int")

    def fetch(self) -> None:
        page1 = self.get_page(1)
        self.max_page = page1.pages

    def add_cache(self, pages: List[Page]):
        for page in pages:
            self.cache[page.number] = page

    @cachedmethod(attrgetter("cache"))
    def get_page(self, page: int) -> Page:
        if page < 1:
            raise KeyError("key harus > 0")
        elif self.max_page and page > self.max_page:
            raise KeyError("key melebihi halaman maksimal")
        params = self.__make_params__(page)
        headers = {"Referer": self.base + "?" + urlencode({"modul": self.subfolder})}
        res = get_url(
            session=self.session,
            url=self.base + "services/view.php",
            params=params,
            username=self.__username__,
            password=self.__password__,
            headers=headers,
        )
        jsonp = res.text[1:-1]
        pages = Page.from_jsonp(jsonp)
        self.add_cache(pages)
        return self.cache[page]

    def __make_params__(self, page: int, format_: str = "jsonp") -> dict:
        return {
            "doc": self.doc,
            "format": format_,
            "subfolder": self.subfolder + "/",
            "page": (page // 10 + 1) * 10,
        }



import logging

from bs4 import BeautifulSoup, Tag
from requests import Session
from typing import Dict, Optional, Mapping

from . import Modul
from .utils import get_url


def parse_doc(href: str) -> str:
    # index.php?subfolder=MSIM4103/&doc=DAFIS.pdf
    return href.split("=")[-1]


class Buku(Mapping[str, Modul]):
    def __init__(
        self,
        kode: str,
        base: str,
        session: Session,
        username: str,
        password: str,
        moduls: Optional[Dict[str, Modul]] = None,
    ):
        self.logger = logging.getLogger(f"Buku:{kode}")
        self.kode = kode
        self.base = base
        self.session = session
        self.username = username
        self.password = password
        if moduls:
            self.moduls = moduls
        else:
            self.moduls = dict()
        if not self.moduls:
            self.fetch()

    def __getitem__(self, key: str):
        return self.moduls[key]

    def __iter__(self):
        return iter(self.moduls)

    def __len__(self):
        return len(self.moduls)

    def fetch(self):
        params = {"modul": self.kode}
        res = get_url(
            session=self.session,
            url=self.base,
            params=params,
            username=self.username,
            password=self.password,
        )
        self.logger.debug("Mencari submodul")
        soup = BeautifulSoup(res.text, "html.parser")
        for th in soup.findAll("th"):
            a: Tag = th.find("a")
            if not a:
                continue
            url = a["href"]
            doc = parse_doc(url)
            self.moduls[doc] = Modul(
                nama=a.getText(),
                subfolder=self.kode,
                doc=doc,
                base=self.base,
                url=url,
                username=self.username,
                password=self.password,
                session=self.session,
            )
            self.logger.debug(f"Dapat submodul {doc}")



import attr
import ujson as json
from typing import List

from . import Font, Text


@attr.dataclass(slots=True)
class Page:
    number: int
    pages: int
    height: int
    width: int
    fonts: List[Font] = attr.ib(factory=list)
    text: List[Text] = attr.ib(factory=list, converter=Text.from_list)  # type: ignore
    texts: str = ""

    @classmethod
    def from_jsonp(cls, jsonp: str) -> List["Page"]:
        pages_data = json.loads(jsonp)
        results: List["Page"] = list()
        for page_data in pages_data:
            results.append(
                Page(
                    number=page_data["number"],
                    pages=page_data["pages"],
                    height=page_data["height"],
                    width=page_data["width"],
                    fonts=page_data["fonts"],
                    text=page_data["text"],
                )
            )
        return results

    def __str__(self) -> str:
        if self.texts:
            return self.texts
        elif not self.text:
            return "Halaman kosong / Tidak ditemukan. :("
        now = 0
        out = ""
        for text in self.text:
            if text.height != now:
                out += "\n"
                now = text.height
            out += str(text)
        self.texts = out
        return out



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['reader_rbv', 'reader_rbv.reader']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.3.0,<21.0.0',
 'beautifulsoup4>=4.9.3,<5.0.0',
 'cachetools>=4.2.1,<5.0.0',
 'requests>=2.25.1,<3.0.0',
 'ujson>=4.0.2,<5.0.0']

setup_kwargs = {
    'name': 'reader-rbv',
    'version': '0.1.0',
    'description': 'SDK / Client python untuk Ruang Baca Virtual Universitas Terbuka.',
    'long_description': '# Reader RBV\n\n[![reader-rbv - PyPi](https://img.shields.io/pypi/v/reader-rbv)](https://pypi.org/project/reader-rbv/)\n[![Supported Python versions](https://img.shields.io/pypi/pyversions/reader-rbv)](https://pypi.org/project/reader-rbv/)\n[![LICENSE](https://img.shields.io/github/license/UnivTerbuka/reader-rbv)](https://github.com/UnivTerbuka/reader-rbv/blob/main/LICENSE)\n[![Code Style - Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\nSDK / Client python untuk Ruang Baca Virtual.\n',
    'author': 'hexatester',
    'author_email': 'habibrohman@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
